// src/index.ts
import SyntaxJSX from "@babel/plugin-syntax-jsx";
import { parse as parse2 } from "@babel/parser";

// src/transform.ts
import { compile } from "@vue-jsx-vapor/compiler";
import generate from "@babel/generator";
import { parse } from "@babel/parser";
import { SourceMapConsumer } from "source-map-js";
import traverse from "@babel/traverse";
var transformJSX = (path, state) => {
  const { parent, node } = path;
  if (!(parent?.type !== "JSXExpressionContainer" && !isJSXElement(parent) && !isConditionalExpression(path.parentPath))) {
    return;
  }
  let { code, vaporHelpers, preamble, map } = compile(
    " ".repeat(node.loc?.start.column || 0) + generate(node).code,
    {
      mode: "module",
      inline: true,
      isTS: state.filename?.endsWith("tsx"),
      filename: state.filename,
      sourceMap: true,
      ...state.opts?.compile
    }
  );
  vaporHelpers.forEach((helper) => state.opts.importSet.add(helper));
  preamble = preamble.replaceAll(
    /(?<=const )t(?=(\d))/g,
    `_t${state.opts.preambleIndex}`
  );
  code = code.replaceAll(/(?<== )t(?=\d)/g, `_t${state.opts.preambleIndex}`).replaceAll("_ctx: any", "").replaceAll("$event: any", "$event");
  state.opts.preambleIndex++;
  for (const [, key, value] of preamble.matchAll(
    /const (_t\d+) = (_template\(.*\))/g
  )) {
    const result = state.opts.preambleMap.get(value);
    if (result) {
      code = code.replaceAll(key, result);
    } else {
      state.opts.preambleMap.set(value, key);
    }
  }
  for (const [, events] of preamble.matchAll(/_delegateEvents\((.*)\)/g)) {
    events.split(", ").forEach((event) => state.opts.delegateEventSet.add(event));
  }
  const ast = parse(code, {
    sourceFilename: state.filename
  });
  if (map) {
    const consumer = new SourceMapConsumer(map);
    const line = (node.loc?.start.line ?? 1) - 1;
    traverse(ast, {
      Identifier({ node: id }) {
        const originalLoc = consumer.originalPositionFor(id.loc.start);
        if (originalLoc) {
          id.loc = {
            ...id.loc,
            start: {
              line: line + originalLoc.line,
              column: originalLoc.column,
              index: originalLoc.column
            },
            end: {
              line: line + originalLoc.line,
              column: originalLoc.column + id.name.length,
              index: originalLoc.column + id.name.length
            }
          };
        }
      }
    });
  }
  path.replaceWith(ast.program.body[0]);
};
function isJSXElement(node) {
  return !!node && (node.type === "JSXElement" || node.type === "JSXFragment");
}
function isConditionalExpression(path) {
  return !!(path && (path?.type === "LogicalExpression" || path.type === "ConditionalExpression") && (path.parent.type === "JSXExpressionContainer" || path.parent.type === "ConditionalExpression" && isConditionalExpression(path.parentPath)));
}

// src/index.ts
var src_default = () => {
  return {
    name: "Vue JSX Vapor",
    inherits: SyntaxJSX,
    visitor: {
      JSXElement: {
        exit: transformJSX
      },
      JSXFragment: {
        exit: transformJSX
      },
      Program: {
        enter: (_, state) => {
          state.opts.importSet = /* @__PURE__ */ new Set();
          state.opts.delegateEventSet = /* @__PURE__ */ new Set();
          state.opts.preambleMap = /* @__PURE__ */ new Map();
          state.opts.preambleIndex = 0;
        },
        exit: (path, { opts: { delegateEventSet, importSet, preambleMap } }) => {
          const statements = [];
          if (delegateEventSet.size) {
            statements.unshift(
              `_delegateEvents(${Array.from(delegateEventSet).join(", ")});`
            );
          }
          if (preambleMap.size) {
            let preambleResult = "";
            for (const [value, key] of preambleMap) {
              preambleResult += `const ${key} = ${value}
`;
            }
            statements.unshift(preambleResult);
          }
          const helpers = ["setText", "createTextNode"].filter((helper) => {
            const result = importSet.has(helper);
            result && importSet.delete(helper);
            return result;
          });
          if (helpers.length) {
            statements.unshift(
              `import { ${helpers.map((i) => `${i} as _${i}`).join(", ")} } from 'unplugin-vue-jsx-vapor/helper.js';
`
            );
          }
          if (importSet.size) {
            const importResult = Array.from(importSet).map((i) => `${i} as _${i}`).join(", ");
            statements.unshift(`import { ${importResult} } from 'vue/vapor';
`);
          }
          path.node.body.unshift(
            ...parse2(statements.join("\n"), { sourceType: "module" }).program.body
          );
        }
      }
    }
  };
};
export {
  src_default as default
};
